/**
 * Main SliderFactory class, contains slider creation logic
 *
 * @param  {sliderArgs}  arguments for the slider object
 *                                        
 * @return {Object}      self
 */
var sliderFactory = function(sliderArgs) {
    var self = this;

    /**
     * Boolean to override loop with false - based on num of slides
     *
     * @type {boolean}
     */
    self.boolLoop = true;


    /**
     * Boolean to override autoplay with 0 - based on num of slides
     *
     * @type {boolean}
     */
    self.boolAutoplay = true;



    /**
     * Primary building logic
     *
     * @param  {sliderArgs} arguments for the slider object passed from obj def
     * @return {self}
     */
    self.buildSlider = function(sliderArgs, numSlides) {



        if (!document.getElementById(sliderArgs.target)) {
            return false;
        }

        sliderArgs.slidesPerView = self.getSlidesPerView(sliderArgs, numSlides);
        sliderArgs.target = "#" + sliderArgs.target;

        if (!self.boolLoop) {
            sliderArgs.loop = false;
        }

        if (!self.boolAutoplay) {
            sliderArgs.autoplay = 0;
        }

        var swiper = new Swiper(sliderArgs.target, self.normalizeArgs(sliderArgs));

        window.addEventListener('resize', function() {
            swiper.params.slidesPerView = self.getSlidesPerView(sliderArgs, numSlides);
            swiper.params.slidesPerGroup = 1;
            swiper.update();
        });

        if (typeof sliderArgs.auto !== 'undefined') {
            swiper.startAutoplay();
        }
        return self;

    };

    /**
     * Primary Takes in sliderArgs and returns arguments appropriate for Swiper 
     * Filters out unsused arguments
     *
     * @param  {sliderArgs} arguments for the slider object passed from obj def
     * @return {swiperArgs} New, cleaned/filtered arguments
     */
    self.normalizeArgs = function(sliderArgs) {

        var swiperArgs = {
            nextButton: sliderArgs.target + ' .swiper-button-next',
            prevButton: sliderArgs.target + ' .swiper-button-prev',
            pagination: sliderArgs.target + ' .swiper-pagination',
            slidesPerView: sliderArgs.slidesPerViewDesktop,
            slidesPerGroup: 1
        };

        // If loader key exists, find loading and add hidden class, and give slider show class
        if (sliderArgs.loader) {
            swiperArgs.onInit = function() {
                var sliderContainer = document.getElementById(sliderArgs.target.slice(1));
                var sliderContainerParent = sliderContainer.parentElement;
                var loader = null;
                for (var i = 0; i < sliderContainerParent.childNodes.length; i++) {
                    if (sliderContainerParent.childNodes[i].className == "loader") {
                        loader = sliderContainerParent.childNodes[i];
                        break;
                    }
                }
                if (loader) {
                    loader.className += " hidden";
                    sliderContainer.className += " show";
                }

            }
        }

        [
            'direction',
            'loop',
            'paginationClickable',
            'spaceBetween',
            'effect',
            'simulateTouch',
            'autoplay',
            'height',
            'autoHeight',
            'onInit',
            'preventClicks',
            'slidesPerView'
        ].forEach(function(strProp) {
            if (sliderArgs.hasOwnProperty(strProp)) {
                swiperArgs[strProp] = sliderArgs[strProp];
            }
        });

        return swiperArgs;

    }

    /**
     * Primary Recieves sliderArgs and number of slides 
     *
     * @param  {sliderArgs} arguments for the slider object passed from obj def
     * @param  {numSlides} original number of slides
     *
     * @return number of slides to be set on slider
     */
    self.getSlidesPerView = function(sliderArgs, numSlides) {
        var windowWidth = $(window).width();
        if (windowWidth < 768) {
            if (numSlides <= sliderArgs.slidesPerViewPhone) {
                this.boolLoop = false;
                this.boolAutoplay = false;
                return numSlides;
            } else {
                return sliderArgs.slidesPerViewPhone;
            }
        } else if (windowWidth < 992) {
            if (numSlides <= sliderArgs.slidesPerViewSmallTablet) {
                this.boolLoop = false;
                this.boolAutoplay = false;
                return numSlides;
            } else {
                return sliderArgs.slidesPerViewSmallTablet;
            }
        } else if (windowWidth < 1200) {
            if (numSlides <= sliderArgs.slidesPerViewLargeTablet) {
                this.boolLoop = false;
                this.boolAutoplay = false;
                return numSlides;
            } else {
                return sliderArgs.slidesPerViewLargeTablet;
            }
        } else {
            if (numSlides <= sliderArgs.slidesPerViewDesktop) {
                this.boolLoop = false;
                this.boolAutoplay = false;
                return numSlides;
            } else {
                return sliderArgs.slidesPerViewDesktop;
            }
        }
    }

    /**
     * Init method
     *
     * @param  {sliderArgs}  arguments for the slider object passed from obj def
     *
     * @return {void}
     */
    self.init = function(sliderArgs) {
        var numSlides = document.querySelectorAll('#' + sliderArgs.target + ' .swiper-slide').length;
        self.buildSlider(sliderArgs, numSlides);
    };


    return self.init(sliderArgs);
}

function handleMainSlider() {
    if ($('#mainSlider').length) {
        $('#mainSlider .bg_link img').each(function() {
            var url = $(this).attr('src');
            $(this).parents('.swiper-slide').css('background-image', 'url("' + url + '")');
        });
        return true;
    } else {
        return false;
    }
}
